
%A MATLAB implementation of
%
%N. Ozay, M. Sznaier, C. Lagoa, O. Camps, "GPCA with Denoising: A
%Moments-Based Convex Approach", CVPR 2010.
%
%Author: Necmiye Ozay <necmiye@coe.neu.edu>
%
%Northeastern University, Boston. September 2009.
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.

%Main test script, generating the teapot example in the paper


clear all;
close all;

init_spectral;

%load teapot data
load teapot2;

num_pnts = size(img1,2);

%figure; plot(img1(1,:),img1(2,:),'b.'); hold on; plot(img2(1,:),img2(2,:),'r.'); axis equal;

[vermap,powers] = perspective_veronese(img1,img2,2*num_object);

[u_n d_n v_n] = svd(vermap);

[powCount sampleCount] = size(vermap);

X_meas = [img1;img2;ones(1,sampleCount)];

momentPowerList = powers(:,1:4);
momentCount = size(momentPowerList,1);

data_prism = zeros(powCount,sampleCount,momentCount);

% the linear relation between the noise free veronese map and the moments
% is:    clean_vermap = sum_{I} moment(I)*data_prism(:,:,I) 

binomialCoefs = pascal(num_object+1,1);
for i = 1:powCount %index of degree (powCount)
    powers_now = powers(i,1:4);
    for j = 1:momentCount %moment count
        moment_now =  momentPowerList(j,:);
        if sum(powers_now >= moment_now) == 4
            for k = 1:4
                dk = powers_now(k);
                mk = moment_now(k);
                if k == 1
                    data_prism(i,:,j) = (binomialCoefs(dk+1,mk+1)).*(X_meas(k,:).^(dk-mk));
                else
                    data_prism(i,:,j) = data_prism(i,:,j).*(binomialCoefs(dk+1,mk+1)).*(X_meas(k,:).^(dk-mk));
                end
            end
        end
    end
end

%%
%calculate the indices to be used in moment matrices
N = num_object;
momNPowerList = momentPowers(4,N);
momNCount = size(momNPowerList,1);
mom2NPowerList = momentPowers(4,2*N);
mom2NCount = size(mom2NPowerList,1);
momNminusPowerList = momentPowers(4,N-1);
momNminusCount = size(momNminusPowerList,1);


%%%%%% "a" is an important parameter
a = 0.1; %noise_level squared
A = a*ones(momNminusCount,momNminusCount);

momentIndices = [];
for i=1:mom2NCount
    dum = ones(powCount,1)*mom2NPowerList(i,:);
    if max(sum(dum==momentPowerList,2))==2*N;
        momentIndices = [momentIndices,i];
    end
end

indSet1 = zeros(momNCount);

for i = 1:momNCount
    for j = 1:momNCount
        sumPow = momNPowerList(i,:)+momNPowerList(j,:);
        xx = (mom2NPowerList == ones(mom2NCount,1)*sumPow);
        indSet1(i,j) = find(sum(xx,2) == 4);
    end
end

indSet2 = zeros(momNminusCount,momNminusCount,4);

for k = 1: 5
    norm_const = zeros(1, 4);
    if k~=5; norm_const(k) = 2;end;
    for i = 1:momNminusCount
        for j = 1:momNminusCount
            sumPow = momNminusPowerList(i,:)+momNminusPowerList(j,:)+norm_const;
            xx = (mom2NPowerList == ones(mom2NCount,1)*sumPow);
            indSet2(i,j,k) = find(sum(xx,2) == 4);
        end
    end
end


%convex optimization problem
delta = 10^-4;
row = powCount; 
col = sampleCount;
Wy = eye(row);
Wz = eye(col);
nnzs = [];

%cvx_quiet(true);
for iter = 1:2 %reweighting iterations
  cvx_solver sedumi
  cvx_begin sdp
    variable Y(row,row) symmetric
    variable Z(col,col) symmetric
    variable Mij(mom2NCount,col)
    X = data_prism(:,:,1);
    for l = 2:momentCount
        X = X+ data_prism(:,:,l).*(ones(row,1)*Mij(momentIndices(l),:));
    end

    minimize( trace(Wy*Y)+trace(Wz*Z))
    subject to
        [Y X; X' Z] >= 0;
        Mij(1,:) == 1;
        for k = 1:col;
            dummy{k} = Mij(:,k);
            dummy{k}(indSet1) >= 0;
            %dummy{k}(indSet2) >= 0;
            dummy2{k} = A.*dummy{k}(indSet2(:,:,5));
            for m = 1:4
                dummy2{k} = dummy2{k} -  dummy{k}(indSet2(:,:,m));
            end
            dummy2{k} >= 0;
        end
    
  cvx_end
  [u d v] =svd(X);
  d=diag(d);
  min_sv(iter) = d(end);
  if min_sv(iter)<10^-7; break; end;
  epsilon = max(delta,d(end));
  Wy = inv(Y+epsilon*eye(row));
  Wz = inv(Z+epsilon*eye(col));
end
cvx_quiet(false);

scale =1;

img1s = img1/scale;
img2s = img2/scale;

noise_est = zeros(4,num_pnts);

ind_first_mom = find(sum(momNPowerList,2)==1);

%find noise estimate
for i=1:num_pnts
    [ut dt vt] = svd(double(dummy{i}(indSet1)));
    temp = ut(:,1)./ut(1,1);
    noise_est(:,i) = temp(ind_first_mom);
end
img1s_dn = img1s-noise_est(1:2,:);
img2s_dn = img2s-noise_est(3:4,:);

%proceed as in HQSA
c = u(:,end);
Jp = perspective_derivative(c,powers,img1s_dn,img2s_dn);

Hp = perspective_hessian(c,powers,img1s_dn,img2s_dn);


simMat = zeros(num_pnts);
for i=1:num_pnts
    for j=i+1:num_pnts
        t12 = null([Jp(:,j)';Jp(:,i)']);
        c12 = t12'*Hp(:,:,i)*t12;
        c21 = t12'*Hp(:,:,j)*t12;
        simMat(i,j) = subspace(c12(:),c21(:));
    end
end
simMat = simMat+simMat';
%figure;
%imshow(simMat,[]);
res=njw_anchor(-simMat,2);

figure;plot(res,'b.'); title('Moments-based Result');
figure;plot(res_true,'r.'); title('Groundtruth Segmentation');
